from distutils.core import setup

setup(
    name = 'torgen',
    packages = ['torgen'], 
    version = '0.2',
    description = 'This package contains Django\'s CBVs adapted to use with Tornado framework.',
    author = 'Alex Parakhnevich',
    author_email = 'alex.parakhnevich@gmail.com',
    url = 'https://github.com/BeardyBear/tornado-generic-handlers',   # use the URL to the github repo
    download_url = 'https://github.com/BeardyBear/tornado-generic-handlers/tarball/0.2', # I'll explain this in a second
    keywords = ['tornado', 'django', 'generic'], # arbitrary keywords
    classifiers = [],
)
